Imports System.Drawing
Imports System.Windows.Forms

'''<summary> Zum Highlighten von Treenodes, ListviewItems etc. </summary>
Public Class Highlighter
   ' Das Highlighten erfolgt durch Farb-Inversion des Target-Rectangles. Ausschalten
   ' des Highlights durch wiederholte Inversion desselben Rectangles. Fehl-Highlighting
   ' ergibt sich also, wenn zwischen Highlight() und Off() das Rectangle anderweitig
   ' bermalt wurde.  

   Protected Shared _HighlightedRect As Rectangle = Rectangle.Empty

   ' Farb-Inversion ist eine Art "Spiegelung" ber eine Farb-Achse.
   ' Dieses komische Rot ergibt (bisher) immer deutliche "Spiegel-Farben" 
   Private Shared ReadOnly _AxisColor As Color = Color.FromArgb(255, 0, 127)

   Public Shared Sub Highlight(ByVal Target As Control, ByVal ItemRect As Rectangle)
      If ItemRect.IsEmpty Then
         Highlighter.Off()
      Else
         ItemRect.Intersect(Target.ClientRectangle)
         ' FillReversibleRectangle verwendet bildschirmbezogene Koordinaten, daher mu ItemRect
         ' um die Bildschirmposition des Controls verschoben werden
         ItemRect.Offset(Target.PointToScreen(Point.Empty))
         If ItemRect.IntersectsWith(_HighlightedRect) Then Return
         ' ist schon highlighted
         If Not _HighlightedRect.IsEmpty Then
            ' altes Highlight lschen
            ControlPaint.FillReversibleRectangle(_HighlightedRect, _AxisColor)
         End If
         _HighlightedRect = ItemRect
         ' neues Highlight setzen
         ControlPaint.FillReversibleRectangle(ItemRect, _AxisColor)
      End If
   End Sub

   Public Shared Sub Before(ByVal Target As Control, ByVal ItemRect As Rectangle)
      ItemRect.Offset(0, -ItemRect.Height \ 2)
      ItemRect.Inflate(0, -2)
      Highlight(Target, ItemRect)
   End Sub

   Public Shared Sub After(ByVal Target As Control, ByVal ItemRect As Rectangle)
      ItemRect.Offset(0, ItemRect.Height)
      Before(Target, ItemRect)
   End Sub

   Public Shared Sub FullWidth(ByVal Target As Control, ByVal Y As Integer, ByVal Height As Integer)
      Highlighter.Highlight(Target, New Rectangle(0, Y, Target.Width, Height))
   End Sub

   Public Shared Sub Off()
      If _HighlightedRect.IsEmpty Then Return
      ControlPaint.FillReversibleRectangle(_HighlightedRect, _AxisColor)
      _HighlightedRect = Rectangle.Empty
   End Sub
End Class