' IDE-Voreinstellungen:
' Option Strict On
' Option Explicit On
' Option Infer On

' Projekt-Voreinstellungen
' Imports System
' Imports System.Drawing
' Imports System.Windows.Forms

Public Class frmMultiDrag

   ' Draggen innerhalb einer Anwendung ist ohne DataObject einfacher und sicherer
   ' Aber Control.DoDragDrop() verlangt eines
   Private Shared _dumData As New DataObject

   ' im Grunde kann bei .DoDragDrop() nur AllEffects angegeben werden, denn beim Drag-Start kann
   ' der AllowedEffekt nicht eingeschrnkt werden: Derselbe Ziehvorgang kann auf dem einen
   ' ZielControl ausschlielich .Copy erfordern, und auf dem anderen ausschlielich .Move.
   ' Es gibt zwar einen DragDropEffects.All, aber wenn man den verwendet, kann 
   ' DragDropEffects.Link nicht eingestellt werden.
   Private Shared _allEffects As DragDropEffects = _
      DragDropEffects.Move Or DragDropEffects.Copy Or DragDropEffects.Link Or DragDropEffects.Scroll

   Private _src, _dest As ListBox
   Private _indxs(-1) As Integer

   Public Sub New()
      InitializeComponent()
      Me.Location = Screen.PrimaryScreen.WorkingArea.Location
   End Sub

   Private Sub ListBox_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) _
         Handles ListBox1.SelectedIndexChanged, ListBox2.SelectedIndexChanged, ListBox3.SelectedIndexChanged
      If _src IsNot Nothing Then Return
      SaveIndicees(DirectCast(sender, ListBox))
   End Sub

   ' ggfs. Dragvorgang starten, und Ergebnis umsetzen (Das _Drop-Event ist fr die Umsetzung der
   ' gemeinten Drag-Aktion ungeeignet, da auftretende Fehler nicht gemeldet werden.)
   Private Sub ListBox_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs) _
         Handles ListBox1.MouseMove, ListBox2.MouseMove, ListBox3.MouseMove
      If e.Button <> Windows.Forms.MouseButtons.Left Then Return
      _src = DirectCast(sender, ListBox)
      ' die durch das MouseDown verlorengegangene MultiSelektion wieder herstellen
      For Each i In _indxs
         _src.SetSelected(i, True)
      Next
      ' vereinfachtes Szenario: Drag-Source von vornherein als Drag-Ziel ausschlieen
      _src.AllowDrop = False
      Try
         Dim effect = _src.DoDragDrop(_dumData, _allEffects)
         If effect = DragDropEffects.None Then Return
         Dim iDest = _dest.IndexFromPoint(_dest.PointToClient(Control.MousePosition))
         If iDest < 0 Then iDest = _dest.Items.Count
         For i = _src.SelectedIndices.Count - 1 To 0 Step -1
            Select Case effect
               Case DragDropEffects.Move
                  _dest.Items.Insert(iDest, _src.SelectedItems(i))
                  _src.Items.RemoveAt(_src.SelectedIndices(i))
               Case DragDropEffects.Copy
                  _dest.Items.Insert(iDest, _src.SelectedItems(i))
               Case DragDropEffects.Link
                  _dest.Items.Insert(iDest, String.Concat("linked with: ", _src.SelectedItems(i)))
            End Select
         Next
      Finally
         ' Aufrum-Arbeiten
         _src.AllowDrop = True
         Highlighter.Off()
         SaveIndicees(_src)
         _src = Nothing
      End Try
   End Sub

   ' Hier wird der DropEffect gesetzt. Dazu zhle ich auch das Highlighten des Ziel-Items
   Private Sub ListBox_DragOver(ByVal sender As Object, ByVal e As DragEventArgs) _
         Handles ListBox1.DragOver, ListBox2.DragOver, ListBox3.DragOver
      If _src Is Nothing Then Return ' Dragvorgnge fremder Anwendungen ablehnen
      _dest = DirectCast(sender, ListBox)
      Select Case Control.ModifierKeys
         Case Keys.None
            e.Effect = DragDropEffects.Move
         Case Keys.Shift
            e.Effect = DragDropEffects.Copy
         Case Keys.Control
            e.Effect = DragDropEffects.Link
         Case Else
            e.Effect = DragDropEffects.None
            Return
      End Select
      Dim pt = New Point(e.X, e.Y)
      If _dest.Items.Count = 0 Then
         Highlighter.FullWidth(_dest, 0, _dest.ItemHeight)
      Else
         Dim i = _dest.IndexFromPoint(_dest.PointToClient(pt))
         If i < 0 Then
            Highlighter.After(_dest, _dest.GetItemRectangle(_dest.Items.Count - 1))
         Else
            Highlighter.Before(_dest, _dest.GetItemRectangle(i))
         End If
      End If
   End Sub

   Private Sub ListBox_DragLeave(ByVal sender As Object, ByVal e As EventArgs) _
         Handles ListBox1.DragLeave, ListBox2.DragLeave, ListBox3.DragLeave
      Highlighter.Off()
   End Sub

   Private Sub SaveIndicees(ByVal lst As ListBox)
      ReDim _indxs(lst.SelectedIndices.Count - 1)
      lst.SelectedIndices.CopyTo(_indxs, 0)
   End Sub

End Class
